/*****************************************************************************\
|*
|*  Version:    1.4
|*
|*  Description:
|*
|*      TASKING M16C ROM Monitor: main routine
|*
|*  Copyright 2003 Altium BV
|*
 *****************************************************************************/

#include <stdio.h>
#include "target.h"
#include "uart.h"
#include "command.h"
#include "m16c.h"

__near unsigned char user_area[USER_AREA_SIZE] __at( USER_AREA_ADDR );

/*****************************************************************************
 *
 *  Command.c variables
 *
 */
__near unsigned char  downloadbuf[DOWNLOADBUFSIZE] __at( DOWNLOAD_BUF_ADDR );
__near unsigned char * __near downloadptr;
__near jmp_buf env;

/* global variables to hold char, int and long command parameters */
__near unsigned long val56;
__near unsigned int  val4;
__near unsigned char val2;

__near char err_code[3];                             /* keep track of the "error" status */
__near t_function cmd_handle;                        /* function pointer to current running command */
#ifdef HAS_4BREAKPOINTS
volatile __near unsigned long break_points[4];       /* internal breakpoints, high bit set: available */
#else
volatile __near unsigned long break_points[2];       /* internal breakpoints, high bit set: available */
#endif

/*****************************************************************************
 *
 *  Uart.c variables
 *
 */
volatile __near unsigned char  se_buf[INPUT_BUFFER_SIZE+1];
volatile __near char           se_buf_in;
volatile __near char           se_buf_out;
volatile __near unsigned char  flag_xoff_sent;

/*****************************************************************************
 *
 *  M16c.c variables
 *
 */
__near unsigned char  user_mode;
__near unsigned char  application_running;
__near unsigned int  * __near isp_monitor;
__near unsigned int  * __near isp_user;

volatile __near unsigned int step_count;
volatile __near unsigned char step_or_go;
volatile __near unsigned long pc_pc;
volatile __near unsigned long pc_rmad0;
volatile __near unsigned long pc_rmad1;
#ifdef HAS_4BREAKPOINTS
volatile __near unsigned long pc_rmad2;
volatile __near unsigned long pc_rmad3;
#endif
__near unsigned long dump_address;

extern __near char _lc_b_main_CLR_DA[];
extern __near char _lc_e_main_CLR_DA[];


/*****************************************************************************
|*
|*  Function:           dummy_user_app
|*
|*  Description:
|*     Functions as a simple user application when the 'GO' command is
|*     issued without any real application being downloaded, or when
|*     the Program Counter is not explicitly set.
|*                      
|*
 */
void dummy_user_app( void )
{
    ta_dummy_user_app();
}

/*****************************************************************************
|*
|*  Function:           _init
|*
|*  Description:        Initialize C variables
|*
|*  This routine replaces the default __init assembly routine in the runtime
|*  library. Reason is that that version is far too big for our purpose.
|*  Instead of using the copy table mechanism we use a more simple method of
|*  first clearing the entire data area and then explicitly setting variables.
|*
|*  Advantage: small initialization routine
|*
|*  Major disadvantage: You can not use explicit initialization on any global
|*  variable, so 'int x = 1' will not work! Instead 'x' has to be set to '1'
|*  in this function. Also all variables need to be declared in this main
|*  module so they all will be placed in the main_CLR_DA section.
|*
 */
void _init( void )
{
    /* first clear all monitor variables */
    __sstrb( 0x0, _lc_b_main_CLR_DA, _lc_e_main_CLR_DA - _lc_b_main_CLR_DA );

    /* now initialize some global variables */
    err_code[0] = '('; err_code[1] = '0'; err_code[2] = '\0';
    break_points[0] |= 0x80000000;
    break_points[1] |= 0x80000000;
#ifdef HAS_4BREAKPOINTS
    break_points[2] |= 0x80000000;
    break_points[3] |= 0x80000000;
#endif
    isp_user = (__near unsigned int *)0x600;

    // application_running = 0;
    // user_mode = 0;

    /* Initialize the user environment, set the user PC and FLG register */
    m16c_set_PC( (unsigned long)dummy_user_app );
    m16c_set_FLG( 0x40 );

    __ldipl( 6 );
}

/*****************************************************************************
|*
|*  Function:           main
|*
|*  Description:
|*      Main routine, initialize the target, then just keep handling all
|*      incoming commands.
 */
void main( void )
{
    ta_init();
    uart_init();

    cmd_show_conf();

    /* loop forever */
    while ( 1 )
    {
        cmd_handle_command();
    }
}
